@@ -1,6 +1,6 @@
-/* $Id: terminal-app.c 24157 2006-12-21 20:53:43Z benny $ */
+/* $Id: terminal-app.c 26424 2007-12-02 14:13:52Z benny $ */
 /*-
- * Copyright (c) 2004-2006 os-cillation e.K.
+ * Copyright (c) 2004-2007 os-cillation e.K.
  *
  * Written by Benedikt Meurer <benny@xfce.org>.
  *
@@ -41,6 +41,7 @@
 #include <terminal/terminal-app.h>
 #include <terminal/terminal-config.h>
 #include <terminal/terminal-preferences.h>
+#include <terminal/terminal-private.h>
 #include <terminal/terminal-window.h>
 
 
@@ -285,7 +286,7 @@ static void
 terminal_app_window_destroyed (GtkWidget   *window,
                                TerminalApp *app)
 {
-  g_return_if_fail (g_list_find (app->windows, window) != NULL);
+  _terminal_return_if_fail (g_list_find (app->windows, window) != NULL);
 
   app->windows = g_list_remove (app->windows, window);
 
@@ -424,7 +425,7 @@ terminal_app_find_screen (const gchar *display_name)
         }
 
       /* mark this screen as handled */
-      g_object_set_data (G_OBJECT (screen), "terminal-checked-screen", GINT_TO_POINTER (1));
+      g_object_set_data (G_OBJECT (screen), I_("terminal-checked-screen"), GINT_TO_POINTER (1));
     }
 #endif
 
@@ -496,8 +497,8 @@ terminal_app_open_window (TerminalApp         *app,
   gchar           *geometry;
   GList           *lp;
 
-  g_return_if_fail (TERMINAL_IS_APP (app));
-  g_return_if_fail (attr != NULL);
+  _terminal_return_if_fail (TERMINAL_IS_APP (app));
+  _terminal_return_if_fail (attr != NULL);
 
   window = terminal_app_create_window (app,
                                        attr->fullscreen,
