@@ -1,6 +1,6 @@
-/* $Id: terminal-screen.c 24157 2006-12-21 20:53:43Z benny $ */
+/* $Id: terminal-screen.c 26424 2007-12-02 14:13:52Z benny $ */
 /*-
- * Copyright (c) 2004-2006 os-cillation e.K.
+ * Copyright (c) 2004-2007 os-cillation e.K.
  *
  * Written by Benedikt Meurer <benny@xfce.org>.
  *
@@ -37,13 +37,11 @@
 #include <unistd.h>
 #endif
 
-#include <exo/exo.h>
-
 #include <terminal/terminal-dialogs.h>
 #include <terminal/terminal-enum-types.h>
-#include <terminal/terminal-helper.h>
 #include <terminal/terminal-image-loader.h>
 #include <terminal/terminal-marshal.h>
+#include <terminal/terminal-private.h>
 #include <terminal/terminal-screen.h>
 #include <terminal/terminal-widget.h>
 
@@ -63,7 +61,6 @@ enum
 enum
 {
   GET_CONTEXT_MENU,
-  OPEN_URI,
   SELECTION_CHANGED,
   LAST_SIGNAL,
 };
@@ -106,10 +103,6 @@ static void       terminal_screen_vte_eof                       (VteTerminal
                                                                  TerminalScreen        *screen);
 static GtkWidget *terminal_screen_vte_get_context_menu          (TerminalWidget        *widget,
                                                                  TerminalScreen        *screen);
-static void       terminal_screen_vte_open_uri                  (TerminalWidget        *widget,
-                                                                 const gchar           *uri,
-                                                                 TerminalHelperCategory category,
-                                                                 TerminalScreen        *screen);
 static void       terminal_screen_vte_selection_changed         (VteTerminal           *terminal,
                                                                  TerminalScreen        *screen);
 static void       terminal_screen_vte_window_title_changed      (VteTerminal           *terminal,
@@ -125,9 +118,6 @@ struct _TerminalScreenClass
 
   /* signals */
   GtkWidget*  (*get_context_menu)  (TerminalScreen        *screen);
-  void        (*open_uri)          (TerminalScreen        *screen,
-                                    const gchar           *uri,
-                                    TerminalHelperCategory category);
   void        (*selection_changed) (TerminalScreen        *screen);
 };
 
@@ -181,8 +171,8 @@ terminal_screen_class_init (TerminalScreenClass *klass)
   g_object_class_install_property (gobject_class,
                                    PROP_CUSTOM_TITLE,
                                    g_param_spec_string ("custom-title",
-                                                        _("Custom title"),
-                                                        _("Custom title"),
+                                                        "custom-title",
+                                                        "custom-title",
                                                         NULL,
                                                         G_PARAM_READWRITE));
 
@@ -192,8 +182,8 @@ terminal_screen_class_init (TerminalScreenClass *klass)
   g_object_class_install_property (gobject_class,
                                    PROP_TITLE,
                                    g_param_spec_string ("title",
-                                                        _("Title"),
-                                                        _("Title"),
+                                                        "title",
+                                                        "title",
                                                         NULL,
                                                         G_PARAM_READABLE));
 
@@ -201,7 +191,7 @@ terminal_screen_class_init (TerminalScreenClass *klass)
    * TerminalScreen::get-context-menu
    **/
   screen_signals[GET_CONTEXT_MENU] =
-    g_signal_new ("get-context-menu",
+    g_signal_new (I_("get-context-menu"),
                   G_TYPE_FROM_CLASS (gobject_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (TerminalScreenClass, get_context_menu),
@@ -209,24 +199,11 @@ terminal_screen_class_init (TerminalScreenClass *klass)
                   _terminal_marshal_OBJECT__VOID,
                   GTK_TYPE_MENU, 0);
 
-  /**
-   * TerminalWidget::open-uri:
-   **/
-  screen_signals[OPEN_URI] =
-    g_signal_new ("open-uri",
-                  G_TYPE_FROM_CLASS (klass),
-                  G_SIGNAL_RUN_LAST,
-                  G_STRUCT_OFFSET (TerminalScreenClass, open_uri),
-                  NULL, NULL,
-                  _terminal_marshal_VOID__STRING_ENUM,
-                  G_TYPE_NONE, 2, G_TYPE_STRING,
-                  TERMINAL_TYPE_HELPER_CATEGORY);
-
   /**
    * TerminalScreen::selection-changed
    **/
   screen_signals[SELECTION_CHANGED] =
-    g_signal_new ("selection-changed",
+    g_signal_new (I_("selection-changed"),
                   G_TYPE_FROM_CLASS (gobject_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (TerminalScreenClass, selection_changed),
@@ -248,7 +225,6 @@ terminal_screen_init (TerminalScreen *screen)
                     "signal::child-exited", G_CALLBACK (terminal_screen_vte_child_exited), screen,
                     "signal::eof", G_CALLBACK (terminal_screen_vte_eof), screen,
                     "signal::context-menu", G_CALLBACK (terminal_screen_vte_get_context_menu), screen,
-                    "signal::open-uri", G_CALLBACK (terminal_screen_vte_open_uri), screen,
                     "signal::selection-changed", G_CALLBACK (terminal_screen_vte_selection_changed), screen,
                     "signal::window-title-changed", G_CALLBACK (terminal_screen_vte_window_title_changed), screen,
                     "swapped-signal::size-allocate", G_CALLBACK (terminal_screen_timer_background), screen,
@@ -485,6 +461,7 @@ static gchar**
 terminal_screen_get_child_environment (TerminalScreen *screen)
 {
   extern gchar **environ;
+  GtkWidget     *toplevel;
   gchar         *display_name;
   gchar        **result;
   gchar        **p;
@@ -525,14 +502,16 @@ terminal_screen_get_child_environment (TerminalScreen *screen)
       g_free (term);
     }
 
-  if (GTK_WIDGET_REALIZED (screen->terminal))
+  /* determine the toplevel widget */
+  toplevel = gtk_widget_get_toplevel (GTK_WIDGET (screen));
+  if (toplevel != NULL && GTK_WIDGET_REALIZED (toplevel))
     {
 #ifdef GDK_WINDOWING_X11
-      result[n++] = g_strdup_printf ("WINDOWID=%ld", (glong) GDK_WINDOW_XWINDOW (screen->terminal->window));
+      result[n++] = g_strdup_printf ("WINDOWID=%ld", (glong) GDK_WINDOW_XWINDOW (toplevel->window));
 #endif
 
       /* determine the DISPLAY value for the command */
-      display_name = gdk_screen_make_display_name (gtk_widget_get_screen (screen->terminal));
+      display_name = gdk_screen_make_display_name (gtk_widget_get_screen (toplevel));
       result[n++] = g_strdup_printf ("DISPLAY=%s", display_name);
       g_free (display_name);
     }
@@ -809,8 +788,8 @@ static void
 terminal_screen_vte_child_exited (VteTerminal    *terminal,
                                   TerminalScreen *screen)
 {
-  g_return_if_fail (VTE_IS_TERMINAL (terminal));
-  g_return_if_fail (TERMINAL_IS_SCREEN (screen));
+  _terminal_return_if_fail (VTE_IS_TERMINAL (terminal));
+  _terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
 
   if (G_LIKELY (!screen->hold))
     gtk_widget_destroy (GTK_WIDGET (screen));
@@ -822,8 +801,8 @@ static void
 terminal_screen_vte_eof (VteTerminal    *terminal,
                          TerminalScreen *screen)
 {
-  g_return_if_fail (VTE_IS_TERMINAL (terminal));
-  g_return_if_fail (TERMINAL_IS_SCREEN (screen));
+  _terminal_return_if_fail (VTE_IS_TERMINAL (terminal));
+  _terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
 
   if (G_LIKELY (!screen->hold))
     gtk_widget_destroy (GTK_WIDGET (screen));
@@ -842,26 +821,12 @@ terminal_screen_vte_get_context_menu (TerminalWidget  *widget,
 
 
 
-static void
-terminal_screen_vte_open_uri (TerminalWidget        *widget,
-                              const gchar           *uri,
-                              TerminalHelperCategory category,
-                              TerminalScreen        *screen)
-{
-  g_return_if_fail (TERMINAL_IS_WIDGET (widget));
-  g_return_if_fail (uri != NULL);
-
-  g_signal_emit (G_OBJECT (screen), screen_signals[OPEN_URI], 0, uri, category);
-}
-
-
-
 static void
 terminal_screen_vte_selection_changed (VteTerminal    *terminal,
                                        TerminalScreen *screen)
 {
-  g_return_if_fail (VTE_IS_TERMINAL (terminal));
-  g_return_if_fail (TERMINAL_IS_SCREEN (screen));
+  _terminal_return_if_fail (VTE_IS_TERMINAL (terminal));
+  _terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
 
   g_signal_emit (G_OBJECT (screen), screen_signals[SELECTION_CHANGED], 0);
 }
@@ -872,8 +837,8 @@ static void
 terminal_screen_vte_window_title_changed (VteTerminal    *terminal,
                                           TerminalScreen *screen)
 {
-  g_return_if_fail (VTE_IS_TERMINAL (terminal));
-  g_return_if_fail (TERMINAL_IS_SCREEN (screen));
+  _terminal_return_if_fail (VTE_IS_TERMINAL (terminal));
+  _terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
 
   g_object_notify (G_OBJECT (screen), "title");
 }
@@ -982,7 +947,7 @@ terminal_screen_launch_child (TerminalScreen *screen)
   gchar  **argv;
   gchar  **env;
 
-  g_return_if_fail (TERMINAL_IS_SCREEN (screen));
+  _terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
 
 #ifdef G_ENABLE_DEBUG
   if (!GTK_WIDGET_REALIZED (screen))
@@ -1025,7 +990,7 @@ void
 terminal_screen_set_custom_command (TerminalScreen *screen,
                                     gchar         **command)
 {
-  g_return_if_fail (TERMINAL_IS_SCREEN (screen));
+  _terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
 
   if (G_UNLIKELY (screen->custom_command != NULL))
     g_strfreev (screen->custom_command);
@@ -1047,7 +1012,7 @@ void
 terminal_screen_set_custom_title (TerminalScreen *screen,
                                   const gchar    *title)
 {
-  g_return_if_fail (TERMINAL_IS_SCREEN (screen));
+  _terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
 
   if (!exo_str_is_equal (screen->custom_title, title))
     {
@@ -1190,7 +1155,7 @@ terminal_screen_get_title (TerminalScreen *screen)
   gchar        *title;
   gchar        *tmp;
 
-  g_return_val_if_fail (TERMINAL_IS_SCREEN (screen), NULL);
+  _terminal_return_val_if_fail (TERMINAL_IS_SCREEN (screen), NULL);
 
   if (G_UNLIKELY (*screen->custom_title != '\0'))
     return g_strdup (screen->custom_title);
@@ -1279,7 +1244,7 @@ terminal_screen_get_working_directory (TerminalScreen *screen)
   gchar *cwd;
   gint   length;
 
-  g_return_val_if_fail (TERMINAL_IS_SCREEN (screen), NULL);
+  _terminal_return_val_if_fail (TERMINAL_IS_SCREEN (screen), NULL);
 
   if (screen->pid >= 0)
     {
@@ -1332,8 +1297,8 @@ void
 terminal_screen_set_working_directory (TerminalScreen *screen,
                                        const gchar    *directory)
 {
-  g_return_if_fail (TERMINAL_IS_SCREEN (screen));
-  g_return_if_fail (directory != NULL);
+  _terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
+  _terminal_return_if_fail (directory != NULL);
 
   g_free (screen->working_directory);
   screen->working_directory = g_strdup (directory);
@@ -1354,7 +1319,7 @@ terminal_screen_set_working_directory (TerminalScreen *screen,
 gboolean
 terminal_screen_get_hold (TerminalScreen *screen)
 {
-  g_return_val_if_fail (TERMINAL_IS_SCREEN (screen), FALSE);
+  _terminal_return_val_if_fail (TERMINAL_IS_SCREEN (screen), FALSE);
   return screen->hold;
 }
 
@@ -1372,7 +1337,7 @@ void
 terminal_screen_set_hold (TerminalScreen *screen,
                           gboolean        hold)
 {
-  g_return_if_fail (TERMINAL_IS_SCREEN (screen));
+  _terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
   screen->hold = hold;
 }
 
@@ -1390,7 +1355,7 @@ terminal_screen_set_hold (TerminalScreen *screen,
 gboolean
 terminal_screen_has_selection (TerminalScreen *screen)
 {
-  g_return_val_if_fail (TERMINAL_IS_SCREEN (screen), FALSE);
+  _terminal_return_val_if_fail (TERMINAL_IS_SCREEN (screen), FALSE);
   return vte_terminal_get_has_selection (VTE_TERMINAL (screen->terminal));
 }
 
@@ -1405,7 +1370,7 @@ terminal_screen_has_selection (TerminalScreen *screen)
 void
 terminal_screen_copy_clipboard (TerminalScreen *screen)
 {
-  g_return_if_fail (TERMINAL_IS_SCREEN (screen));
+  _terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
   vte_terminal_copy_clipboard (VTE_TERMINAL (screen->terminal));
 }
 
@@ -1422,7 +1387,7 @@ terminal_screen_copy_clipboard (TerminalScreen *screen)
 void
 terminal_screen_paste_clipboard (TerminalScreen *screen)
 {
-  g_return_if_fail (TERMINAL_IS_SCREEN (screen));
+  _terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
   vte_terminal_paste_clipboard (VTE_TERMINAL (screen->terminal));
 }
 
@@ -1440,7 +1405,7 @@ terminal_screen_paste_clipboard (TerminalScreen *screen)
 void
 terminal_screen_paste_primary (TerminalScreen *screen)
 {
-  g_return_if_fail (TERMINAL_IS_SCREEN (screen));
+  _terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
   vte_terminal_paste_primary (VTE_TERMINAL (screen->terminal));
 }
 
@@ -1457,7 +1422,7 @@ void
 terminal_screen_reset (TerminalScreen *screen,
                        gboolean        clear)
 {
-  g_return_if_fail (TERMINAL_IS_SCREEN (screen));
+  _terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
   vte_terminal_reset (VTE_TERMINAL (screen->terminal), TRUE, clear);
 }
 
@@ -1476,8 +1441,8 @@ void
 terminal_screen_im_append_menuitems (TerminalScreen *screen,
                                      GtkMenuShell   *menushell)
 {
-  g_return_if_fail (TERMINAL_IS_SCREEN (screen));
-  g_return_if_fail (GTK_IS_MENU_SHELL (menushell));
+  _terminal_return_if_fail (TERMINAL_IS_SCREEN (screen));
+  _terminal_return_if_fail (GTK_IS_MENU_SHELL (menushell));
 
   vte_terminal_im_append_menuitems (VTE_TERMINAL (screen->terminal), menushell);
 }
@@ -1496,7 +1461,7 @@ terminal_screen_get_restart_command (TerminalScreen *screen)
   const gchar *directory;
   GList       *result = NULL;
 
-  g_return_val_if_fail (TERMINAL_IS_SCREEN (screen), NULL);
+  _terminal_return_val_if_fail (TERMINAL_IS_SCREEN (screen), NULL);
 
   if (screen->custom_command != NULL)
     {
